<?php


abstract class AbstractQuickPayControllerAdmin extends Controller
{
    /**
     * @var null
     */
    protected $module_name = NULL;

    /**
     * @var array
     */
    protected $data = array();

    /**
     * @var null
     */
    protected $error = NULL;

    /**
     * Instantiation logic on controller construction
     */
    protected function init()
    {
        $this->language->load('payment/' . $this->module_name);

        $this->document->setTitle($this->language->get('heading_title'));

        $this->load->model('setting/setting');

        if (($this->request->server['REQUEST_METHOD'] == 'POST') && ($this->validate())) {
            $this->model_setting_setting->editSetting($this->module_name, $this->request->post);

            $this->session->data['success'] = $this->language->get('text_success');

            $this->response->redirect($this->url->link('payment/' . $this->module_name, 'token=' . $this->session->data['token'], 'SSL'));
        }
    }

    /**
     * Validates the user permissions
     *
     * @return bool
     */
    protected function validate()
    {
        if (!$this->user->hasPermission('modify', sprintf('payment/%s', $this->module_name))) {
            $this->error['warning'] = $this->language->get('error_permission');
        }

        $errors = array('merchant', 'agreement', 'apikey', 'privatekey');
        foreach ($errors as $error) {
            if (!$this->request->post[$this->module_name . '_' . $error]) {
                $this->error[$error] = $this->language->get('error_' . $error);
            }
        }

        return !$this->error;
    }

    /**
     * Convenience wrapper to retrieve settings
     *
     * @param $field
     * @param string $default
     * @return string
     */
    protected function get_setting($field, $default = '')
    {
        $data = $this->config->get($field);

        if (!empty($data)) {
            return $data;
        }

        return $default;
    }

    /**
     * Populates a data array.
     */
    protected function populate_common_data()
    {
        // Define keys
        $keys = array(
            'heading_title',
            'text_edit',
            'text_enabled',
            'text_disabled',
            'text_all_zones',
            'text_yes',
            'text_no',
            'text_authorization',
            'text_sale',
            'entry_status',
            'entry_apikey',
            'entry_privatekey',
            'entry_cardtypelock',
            'entry_agreement',
            'entry_autofee',
            'entry_msgtype',
            'entry_merchant',
            'entry_autocapture',
            'entry_order_status_completed',
            'entry_text_on_statement',
            'entry_branding_id',
            'tooltip_status',
            'tooltip_merchant',
            'tooltip_agreement',
            'tooltip_apikey',
            'tooltip_privatekey',
            'tooltip_msgtype',
            'tooltip_autocapture',
            'tooltip_cardtypelock',
            'tooltip_autofee',
            'tooltip_order_status_completed',
            'tooltip_status',
            'tooltip_text_on_statement',
            'tooltip_branding_id',
            'button_save',
            'button_cancel',
        );

        // Populate
        foreach ($keys as $key) {
            $this->data[$key] = $this->language->get($key);
        }

        // Extras
        $this->load->model('localisation/order_status');
        $this->data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();

        $this->data['header'] = $this->load->controller('common/header');
        $this->data['column_left'] = $this->load->controller('common/column_left');
        $this->data['footer'] = $this->load->controller('common/footer');
    }

    /**
     * Populates breadcrum data to the data array
     */
    protected function populate_common_data_breadcrumb()
    {
        $this->data['breadcrumbs'] = array();

        $this->data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/home', 'token=' . $this->session->data['token'], 'SSL'),
            'separator' => false
        );

        $this->data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_payment'),
            'href' => $this->url->link('payment', 'token=' . $this->session->data['token'], 'SSL'),
            'separator' => ' :: '
        );

        $this->data['breadcrumbs'][] = array(
            'text' => $this->language->get('heading_title'),
            'href' => $this->url->link('payment/' . $this->module_name, 'token=' . $this->session->data['token'], 'SSL'),
            'separator' => ' :: '
        );

        $this->data['action'] = $this->url->link('payment/' . $this->module_name, 'token=' . $this->session->data['token'], 'SSL');

        $this->data['cancel'] = $this->url->link('payment/', 'token=' . $this->session->data['token'], 'SSL');
    }

    /**
     * Auto-populates the field data
     */
    protected function get_config_data()
    {
        // Define settings fields
        $fields = array(
            'status',
            'merchant',
            'agreement',
            'privatekey',
            'apikey',
            'secret',
            'order_status_completed',
            'autocapture',
            'autofee',
            'text_on_statement',
            'branding_id',
        );

        // Contains default values
        $defaults = array(
            'status' => '0',
            'order_status_completed' => '5',
            'autocapture' => '0',
            'autofee' => '0',
            'text_on_statement' => '',
            'branding_id' => ''
        );

        // Loop through configuration fields and populate them
        foreach ($fields as $field) {
            $field = $this->module_name . '_' . $field;
            if (isset($this->request->post[$field])) {
                $this->data[$field] = $this->request->post[$field];
            } else {
                $this->data[$field] = $this->config->get($field);
            }
        }

        // Check if fields with required default data is set. If not, we will populate default data to them.
        foreach ($defaults as $field => $default_value) {
            $field = $this->module_name . '_' . $field;
            if (!isset($this->data[$field])) {
                $this->data[$field] = $default_value;
            }
        }
    }

    /**
     * Sets error messages
     * @param array $error_fields
     */
    protected function populate_error_messages()
    {
        $error_fields = $this->get_error_fields();

        if (!empty($error_fields)) {
            foreach ($error_fields as $error) {
                if (isset($this->error[$error])) {
                    $this->data['error_' . $error] = $this->error[$error];
                } else {
                    $this->data['error_' . $error] = '';
                }
            }
        }
    }

    /**
     * Returns the fields we should set error messages for.
     *
     * @return array
     */
    abstract protected function get_error_fields();

}